@@ -1,5 +1,5 @@
 /* info-utils.c -- miscellanous.
-   $Id: info-utils.c,v 1.2 2003/03/06 23:22:23 karl Exp $
+   $Id: info-utils.c,v 1.3 2004/02/05 00:29:29 dirt Exp $
 
    Copyright (C) 1993, 1998, 2003 Free Software Foundation, Inc.
 
@@ -37,6 +37,10 @@ char *info_parsed_filename = (char *)NULL;
    calling info_parse_xxx (). */
 char *info_parsed_nodename = (char *)NULL;
 
+/* Variable which holds the most recent line number parsed as a result of
+   calling info_parse_xxx (). */
+int info_parsed_line_number = 0;
+
 /* Functions to remember a filename or nodename for later return. */
 static void save_filename (), saven_filename ();
 static void save_nodename (), saven_nodename ();
@@ -96,6 +100,33 @@ info_parse_node (string, newlines_okay)
       free (info_parsed_nodename);
       info_parsed_nodename = (char *)NULL;
     }
+
+  /* Parse ``(line ...)'' part of menus, if any.  */
+  {
+    char *rest = (char *) (string + i + 1);
+
+    /* Skip any whitespace first, and then a newline in case the item
+       was so long to contain the ``(line ...)'' string in the same
+       physical line.  */
+    while (whitespace(*rest))
+      rest++;
+    if (*rest == '\n')
+      {
+        rest++;
+        while (whitespace(*rest))
+          rest++;
+      }
+
+    /* Are we looking at an opening parenthesis?  That can only mean
+       we have a winner. :)  */
+    if (strncmp (rest, "(line ", strlen ("(line ")) == 0)
+      {
+        rest += strlen ("(line ");
+        info_parsed_line_number = strtol (rest, NULL, 0);
+      }
+    else
+      info_parsed_line_number = 0;
+  }
 }
 
 /* Return the node addressed by LABEL in NODE (usually one of "Prev:",
@@ -290,6 +321,8 @@ info_references_internal (label, binding)
 
           if (info_parsed_nodename)
             entry->nodename = xstrdup (info_parsed_nodename);
+
+          entry->line_number = info_parsed_line_number;
         }
 
       add_pointer_to_array
