@@ -1,8 +1,8 @@
 /* info.c -- Display nodes of Info files in multiple windows.
-   $Id: info.c,v 1.60 2002/03/11 19:54:29 karl Exp $
+   $Id: info.c,v 1.6 2004/02/24 03:25:46 dirt Exp $
 
-   Copyright (C) 1993, 96, 97, 98, 99, 2000, 01, 02
-   Free Software Foundation, Inc.
+   Copyright (C) 1993, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
+   2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -77,8 +77,11 @@ int dump_subnodes = 0;
 /* Non-zero means make default keybindings be loosely modeled on vi(1).  */
 int vi_keys_p = 0;
 
-/* Non-zero means don't remove ANSI escape sequences from man pages.  */
-int raw_escapes_p = 0;
+/* Non-zero means don't remove ANSI escape sequences.  */
+int raw_escapes_p = 1;
+
+/* Non-zero means print the absolute location of the file to be loaded.  */
+static int print_where_p = 0;
 
 #ifdef __MSDOS__
 /* Non-zero indicates that screen output should be made 'speech-friendly'.
@@ -103,15 +106,18 @@ static struct option long_options[] = {
   { "file", 1, 0, 'f' },
   { "help", 0, &print_help_p, 1 },
   { "index-search", 1, 0, IDXSRCH_OPTION },
+  { "location", 0, &print_where_p, 1 },
   { "node", 1, 0, 'n' },
   { "output", 1, 0, 'o' },
   { "raw-escapes", 0, &raw_escapes_p, 1 },
+  { "no-raw-escapes", 0, &raw_escapes_p, 0 },
   { "restore", 1, 0, RESTORE_OPTION },
   { "show-options", 0, 0, 'O' },
   { "subnodes", 0, &dump_subnodes, 1 },
   { "usage", 0, 0, 'O' },
   { "version", 0, &print_version_p, 1 },
   { "vi-keys", 0, &vi_keys_p, 1 },
+  { "where", 0, &print_where_p, 1 },
 #ifdef __MSDOS__
   { "speech-friendly", 0, &speech_friendly, 1 },
 #endif
@@ -120,16 +126,16 @@ static struct option long_options[] = {
 
 /* String describing the shorthand versions of the long options found above. */
 #ifdef __MSDOS__
-static char *short_options = "d:n:f:o:ORsb";
+static char *short_options = "d:n:f:ho:ORswb";
 #else
-static char *short_options = "d:n:f:o:ORs";
+static char *short_options = "d:n:f:ho:ORws";
 #endif
 
 /* When non-zero, the Info window system has been initialized. */
 int info_windows_initialized_p = 0;
 
 /* Some "forward" declarations. */
-static void info_short_help (), remember_info_program_name ();
+static void info_short_help ();
 static void init_messages ();
 extern void add_file_directory_to_path ();
 
@@ -158,7 +164,7 @@ main (argc, argv)
   textdomain (PACKAGE);
 
   init_messages ();
-  
+
   while (1)
     {
       int option_character;
@@ -166,7 +172,7 @@ main (argc, argv)
       option_character = getopt_long
         (argc, argv, short_options, long_options, &getopt_long_index);
 
-      /* getopt_long () returns EOF when there are no more long options. */
+      /* getopt_long returns EOF when there are no more long options. */
       if (option_character == EOF)
         break;
 
@@ -199,6 +205,11 @@ main (argc, argv)
           user_filename = xstrdup (optarg);
           break;
 
+          /* Treat -h like --help. */
+        case 'h':
+          print_help_p = 1;
+          break;
+
           /* User is specifying the name of a file to output to. */
         case 'o':
           if (user_output_filename)
@@ -224,6 +235,11 @@ main (argc, argv)
           dump_subnodes = 1;
           break;
 
+          /* For compatibility with man, -w is --where.  */
+        case 'w':
+          print_where_p = 1;
+          break;
+
 #ifdef __MSDOS__
 	  /* User wants speech-friendly output.  */
 	case 'b':
@@ -276,11 +292,10 @@ main (argc, argv)
     {
       printf ("%s (GNU %s) %s\n", program_name, PACKAGE, VERSION);
       puts ("");
-      printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
-There is NO warranty.  You may redistribute this software\n\
+      puts ("Copyright (C) 2004 Free Software Foundation, Inc.");
+      printf (_("There is NO warranty.  You may redistribute this software\n\
 under the terms of the GNU General Public License.\n\
-For more information about these matters, see the files named COPYING.\n"),
-		  "2002");
+For more information about these matters, see the files named COPYING.\n"));
       xexit (0);
     }
 
@@ -317,7 +332,13 @@ For more information about these matters, see the files named COPYING.\n"),
         {
           info_add_path (DEFAULT_INFOPATH, INFOPATH_PREPEND);
 #ifdef INFODIR /* from the Makefile */
-         info_add_path (INFODIR, INFOPATH_PREPEND);
+          info_add_path (INFODIR, INFOPATH_PREPEND);
+#endif
+#ifdef INFODIR2 /* from the Makefile, too */
+#  ifdef INFODIR
+          if (!STREQ (INFODIR, INFODIR2))
+#  endif
+            info_add_path (INFODIR2, INFOPATH_PREPEND);
 #endif
         }
     }
@@ -359,7 +380,9 @@ For more information about these matters, see the files named COPYING.\n"),
     {
       free (initial_node);
 
-      if (user_output_filename)
+      if (print_where_p)
+        printf ("%s\n", user_filename ? user_filename : "unknown?!");
+      else if (user_output_filename)
         dump_nodes_to_file
           (user_filename, user_nodenames, user_output_filename, dump_subnodes);
       else
@@ -380,6 +403,18 @@ For more information about these matters, see the files named COPYING.\n"),
     if (new_initial_node && new_initial_node != initial_node)
       initial_node = new_initial_node;
 
+    if (print_where_p)
+      {
+        if (initial_node->parent)
+          printf ("%s\n", initial_node->parent);
+        else if (initial_node->filename
+            && !is_dir_name (filename_non_directory (initial_node->filename)))
+          printf ("%s\n", initial_node->filename);
+        else
+          xexit (1);
+        xexit (0);
+      }
+
     /* If the user specified that this node should be output, then do that
        now.  Otherwise, start the Info session with this node.  Or act
        accordingly if the initial node was not found.  */
@@ -473,6 +508,8 @@ For more information about these matters, see the files named COPYING.\n"),
 
     xexit (0);
   }
+
+  return 0; /* Avoid bogus warnings.  */
 }
 
 void
@@ -558,7 +595,7 @@ info_short_help ()
   static const char speech_friendly_string[] = "";
 #endif
 
-    
+
   printf (_("\
 Usage: %s [OPTION]... [MENU-ITEM...]\n\
 \n\
@@ -573,10 +610,12 @@ Options:\n\
       --index-search=STRING    go to node pointed by index entry STRING.\n\
   -n, --node=NODENAME          specify nodes in first visited Info file.\n\
   -o, --output=FILENAME        output selected nodes to FILENAME.\n\
-  -R, --raw-escapes            don't remove ANSI escapes from man pages.\n\
+  -R, --raw-escapes            output \"raw\" ANSI escapes (default).\n\
+      --no-raw-escapes         output escapes as literal text.\n\
       --restore=FILENAME       read initial keystrokes from FILENAME.\n\
   -O, --show-options, --usage  go to command-line options node.\n%s\
       --subnodes               recursively output menu items.\n\
+  -w, --where, --location      print physical location of Info file.\n\
       --vi-keys                use vi-like and less-like key bindings.\n\
       --version                display version information and exit.\n\
 \n\
@@ -609,22 +648,22 @@ Texinfo home page: http://www.gnu.org/software/texinfo/"));
    use them that way.  This also has the advantage that there's only one
    copy of the strings.  */
 
-char *msg_cant_find_node;
-char *msg_cant_file_node;
-char *msg_cant_find_window;
-char *msg_cant_find_point;
-char *msg_cant_kill_last;
-char *msg_no_menu_node;
-char *msg_no_foot_node;
-char *msg_no_xref_node;
-char *msg_no_pointer;
-char *msg_unknown_command;
-char *msg_term_too_dumb;
-char *msg_at_node_bottom;
-char *msg_at_node_top;
-char *msg_one_window;
-char *msg_win_too_small;
-char *msg_cant_make_help;
+const char *msg_cant_find_node;
+const char *msg_cant_file_node;
+const char *msg_cant_find_window;
+const char *msg_cant_find_point;
+const char *msg_cant_kill_last;
+const char *msg_no_menu_node;
+const char *msg_no_foot_node;
+const char *msg_no_xref_node;
+const char *msg_no_pointer;
+const char *msg_unknown_command;
+const char *msg_term_too_dumb;
+const char *msg_at_node_bottom;
+const char *msg_at_node_top;
+const char *msg_one_window;
+const char *msg_win_too_small;
+const char *msg_cant_make_help;
 
 static void
 init_messages ()
