@@ -1,7 +1,8 @@
 /*  man.c: How to read and format man files.
-    $Id: man.c,v 1.13 1999/07/05 20:43:23 karl Exp $
+    $Id: man.c,v 1.1.1.1 2003/12/06 01:32:53 karl Exp $
 
-   Copyright (C) 1995, 97, 98, 99 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1997, 1998, 1999, 2000, 2002, 2003 Free Software
+   Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -159,7 +160,7 @@ create_manpage_file_buffer ()
   file_buffer->filesize = 0;
   file_buffer->contents = (char *)NULL;
   file_buffer->flags = (N_IsInternal | N_CannotGC | N_IsManPage);
-  
+
   return (file_buffer);
 }
 
@@ -397,11 +398,36 @@ clean_manpage (manpage)
           newline_count--;
         }
 
-      if (manpage[i] == '\b' || manpage[i] == '\f')
+      /* A malformed man page could have a \b as its first character,
+         in which case decrementing j by 2 will cause us to write into
+         newpage[-1], smashing the hidden info stored there by malloc.  */
+      if (manpage[i] == '\b' || (manpage[i] == '\f' && j > 0))
         j -= 2;
+      else if (!raw_escapes_p)
+	{
+	  /* Remove the ANSI escape sequences for color, boldface,
+	     underlining, and italics, generated by some versions of
+	     Groff.  */
+	  if (manpage[i] == '\033' && manpage[i + 1] == '['
+	      && isdigit (manpage[i + 2]))
+	    {
+	      if (isdigit (manpage[i + 3]) && manpage[i + 4] == 'm')
+		{
+		  i += 4;
+		  j--;
+		}
+	      else if (manpage[i + 3] == 'm')
+		{
+		  i += 3;
+		  j--;
+		}
+	      /* Else do nothing: it's some unknown escape sequence,
+		 so let's leave it alone.  */
+	    }
+	}
     }
 
-  newpage[j++] = '\0';
+  newpage[j++] = 0;
 
   strcpy (manpage, newpage);
   free (newpage);
